#!/bin/bash

set -e 

check_dependencies() {
	if ! which i686-w64-mingw32-gcc >/dev/null ; then
		echo "FATAL: Missing i686-w64-mingw32-gcc" >&2
		exit 1
	fi
	
	if ! which windres >/dev/null ; then
		echo "FATAL: Missing binutils::windres" >&2
		exit 1
	fi
}

c_escape() {
	sed -r 's~(["\\])~\\\1~g'
}

makecfile() {
	local embed=$(echo "$1" | c_escape)
	cat <<EOD
#include <Windows.h>

int CALLBACK WinMain(HINSTANCE a, HINSTANCE b, LPSTR c, int d) {
	int ret = (int)ShellExecuteW(NULL, NULL, L"$embed", NULL, NULL, SW_SHOWNORMAL);
	return (ret > 32) ? 0 : 1;
}
EOD
}

makeresfile() {
	if [[ -f "$1" ]] ; then
		local embed=$(echo "$1" | c_escape)
		
		local rcfile=$(mktemp --suffix=.rc)
		echo "id ICON \"$embed\"" > "$rcfile"
		
		local resfile=$(mktemp --suffix=.res)
		windres "$rcfile" -O coff -o "$resfile"
		rm "$rcfile"
		
		echo "$resfile"
	else
		echo "" #return nothing
	fi
}

makeexe() {
	
	# icon file
	local resfile=$(makeresfile "$3")
	
	# c file
	local cfile=$(mktemp --suffix=.c)
	makecfile "$1" > "$cfile"
	
	# should be possible to reduce binary size further with -nostartupfiles etc
	i686-w64-mingw32-gcc -Wl,-subsystem,windows -s -Os "$cfile" $resfile -o "$2"
	
	# tidy up
	rm "$cfile"
	if [[ -f "$resfile" ]] ; then
		rm "$resfile"
	fi
}

main() {
	check_dependencies
	
	if [[ $# == 2 ]] ; then
		makeexe "$1" "$2" ""
		
	elif [[ $# == 3 ]] ; then
		if [[ ! -f "$2" ]] ; then
			echo "WARNING: Couldn't read icon file \"$2\"" >&2
		fi
		makeexe "$1" "$3" "$2"
	
	else
		echo "Usage: cmd2exe 'windows command' [icon.ico] output.exe" >&2
		exit 1
	fi
}

main "$@"
